package be.iminds.ilabt.jfed.rspec;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.*;

/**
 * config for AdvertisementBasedRspecGenerator
 */
public class AdvertisementBasedRspecGeneratorConfig {
    @Nonnull public final List<NodeInfo> nodes;

    @JsonCreator
    public AdvertisementBasedRspecGeneratorConfig(
            @Nonnull @JsonProperty("nodes") List<NodeInfo> nodes) {
        this.nodes = nodes;
    }

    @Nonnull
    public List<NodeInfo> getNodes() {
        return nodes;
    }

    public static class NodeInfo {
        public final int count;
        @Nonnull public final String clientIdBase;
        @Nullable public final String ansibleGroup;
        @Nullable public final String diskImage;
        @Nullable public final List<String> componentId;
        @Nullable public final String sliverType;
        @Nullable public final List<String> hardwareTypeFilter;
        @Nullable public final Boolean availableFilter;
        @Nullable public final Boolean exclusive;

        public NodeInfo(          int count,
                        @Nullable String clientIdBase,
                        @Nullable String ansibleGroup,
                        @Nullable String diskImage,
                        @Nullable List<String> componentId,
                        @Nullable String sliverType,
                        @Nullable List<String> hardwareTypeFilter,
                        @Nullable Boolean availableFilter,
                        @Nullable Boolean exclusive) {
            this.count = count;
            this.clientIdBase = clientIdBase == null ? "node" : clientIdBase;
            this.ansibleGroup = ansibleGroup;
            this.diskImage = diskImage;
            this.componentId = componentId;
            this.sliverType = sliverType;
            this.hardwareTypeFilter = hardwareTypeFilter;
            this.availableFilter = availableFilter;
            this.exclusive = exclusive;
        }

        @JsonCreator
        public static NodeInfo createNodeInfoFromJsonObject(@Nonnull Map<String,Object> props) {
            //TODO: Check if types are correct etc
            int count = (int) props.get("count");
            List<String> componentId = null;
            Object componentIdObj = props.get("componentId");
            if (componentIdObj != null) {
                if (componentIdObj instanceof Collection) {
                    componentId = new ArrayList((Collection) componentIdObj);
                }
                if (componentIdObj instanceof String) {
                    componentId = Collections.singletonList((String) componentIdObj);
                }
                assert componentId != null;
            }
            List<String> hardwareTypeFilter = null;
            Object hardwareTypeFilterObj = props.get("hardwareTypeFilter");
            if (hardwareTypeFilterObj != null) {
                if (hardwareTypeFilterObj instanceof Collection) {
                    hardwareTypeFilter = new ArrayList((Collection) hardwareTypeFilterObj);
                }
                if (hardwareTypeFilterObj instanceof String) {
                    hardwareTypeFilter = Collections.singletonList((String) hardwareTypeFilterObj);
                }
                assert hardwareTypeFilter != null;
            }
            return new NodeInfo(count,
                    (String) props.get("clientIdBase"),
                    (String) props.get("ansibleGroup"),
                    (String) props.get("diskImage"),
                    componentId,
                    (String) props.get("sliverType"),
                    hardwareTypeFilter,
                    (Boolean) props.get("availableFilter"),
                    (Boolean) props.get("exclusive"));
        }

        public int getCount() {
            return count;
        }

        @Nonnull
        public String getClientIdBase() {
            return clientIdBase;
        }

        @Nullable
        public String getAnsibleGroup() {
            return ansibleGroup;
        }

        @Nullable
        public String getDiskImage() {
            return diskImage;
        }

        @Nullable
        public List<String> getComponentId() {
            return componentId;
        }

        @Nullable
        public String getSliverType() {
            return sliverType;
        }

        @Nullable
        public List<String> getHardwareTypeFilter() {
            return hardwareTypeFilter;
        }

        @Nullable
        public Boolean getAvailableFilter() {
            return availableFilter;
        }

        @Nullable
        public Boolean getExclusive() {
            return exclusive;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof NodeInfo)) return false;

            NodeInfo nodeInfo = (NodeInfo) o;

            if (count != nodeInfo.count) return false;
            if (!clientIdBase.equals(nodeInfo.clientIdBase)) return false;
            if (ansibleGroup != null ? !ansibleGroup.equals(nodeInfo.ansibleGroup) : nodeInfo.ansibleGroup != null)
                return false;
            if (diskImage != null ? !diskImage.equals(nodeInfo.diskImage) : nodeInfo.diskImage != null) return false;
            if (componentId != null ? !componentId.equals(nodeInfo.componentId) : nodeInfo.componentId != null)
                return false;
            if (sliverType != null ? !sliverType.equals(nodeInfo.sliverType) : nodeInfo.sliverType != null)
                return false;
            if (hardwareTypeFilter != null ? !hardwareTypeFilter.equals(nodeInfo.hardwareTypeFilter) : nodeInfo.hardwareTypeFilter != null)
                return false;
            if (availableFilter != null ? !availableFilter.equals(nodeInfo.availableFilter) : nodeInfo.availableFilter != null)
                return false;
            return exclusive != null ? exclusive.equals(nodeInfo.exclusive) : nodeInfo.exclusive == null;
        }

        @Override
        public int hashCode() {
            int result = count;
            result = 31 * result + clientIdBase.hashCode();
            result = 31 * result + (ansibleGroup != null ? ansibleGroup.hashCode() : 0);
            result = 31 * result + (diskImage != null ? diskImage.hashCode() : 0);
            result = 31 * result + (componentId != null ? componentId.hashCode() : 0);
            result = 31 * result + (sliverType != null ? sliverType.hashCode() : 0);
            result = 31 * result + (hardwareTypeFilter != null ? hardwareTypeFilter.hashCode() : 0);
            result = 31 * result + (availableFilter != null ? availableFilter.hashCode() : 0);
            result = 31 * result + (exclusive != null ? exclusive.hashCode() : 0);
            return result;
        }

        @Override
        public String toString() {
            return "NodeInfo{" +
                    "count=" + count +
                    ", clientIdBase='" + clientIdBase + '\'' +
                    ", ansibleGroup='" + ansibleGroup + '\'' +
                    ", diskImage='" + diskImage + '\'' +
                    ", componentId=" + componentId +
                    ", sliverType='" + sliverType + '\'' +
                    ", hardwareTypeFilter=" + hardwareTypeFilter +
                    ", availableFilter=" + availableFilter +
                    ", exclusive=" + exclusive +
                    '}';
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof AdvertisementBasedRspecGeneratorConfig)) return false;

        AdvertisementBasedRspecGeneratorConfig that = (AdvertisementBasedRspecGeneratorConfig) o;

        return nodes.equals(that.nodes);
    }

    @Override
    public int hashCode() {
        return nodes.hashCode();
    }

    @Override
    public String toString() {
        return "AdvertisementBasedRspecGeneratorConfig{" +
                "nodes=" + nodes +
                '}';
    }
}
